package P4;

use strict;

use vars qw(@ISA @verbs);
use subs qw(import AUTOLOAD);

require Exporter;
@ISA = qw(Exporter);

@verbs = qw(
 add admin branch branches change changes client clients counter counters
 delete depot depots describe diff diff2 dirs edit filelog files fix
 fixes flush fstat group groups have help info integrate integrated job
 jobs jobspec label labels labelsync lock obliterate opened passwd print
 protect rename reopen resolve resolved revert review reviews set submit
 sync triggers unlock user users verify where 
);

sub import {
	my $self = shift;
	my ($callpack, $callfile, $callline) = caller;
	my @EXPORT = 'AUTOLOAD';

	foreach my $sym (@EXPORT) {
no strict "refs";
		*{"${callpack}::$sym"} = \&{"P4::$sym"};
use strict "refs";
	}
};

AUTOLOAD {
	use vars qw(*AUTOLOAD);
	my $cmd = $AUTOLOAD;
	$cmd =~ s/^.*:://;

	if (grep /$cmd/, @verbs) {
		my ($callpack, $callfile, $callline) = caller;
no strict "refs";
		my $opts = ${"${callpack}::options"};
		my $args = ${"${callpack}::arguments"};
use strict "refs";

	    eval qq {
			*$AUTOLOAD = sub {
				open(SUBPROC, "p4 $opts $cmd $args |")
						or die "Can't exec $cmd: \$!\n";
				if (wantarray) {
					my \@ret = <SUBPROC>;
					close SUBPROC;
					\@ret;
				} else {
					local(\$/) = undef;
					my \$ret = <SUBPROC>;
					close SUBPROC;
					\$ret;
				}
			}
		};
	} else {
	    eval qq {
			*$AUTOLOAD = sub { P4::Simple::$cmd(\@_); }
		};
	}
	goto &$AUTOLOAD;
}

package P4::Simple;

use strict;
use Carp;

#require Exporter;

local (*STDOUT, *IN, *OUT);

use subs qw(
 run_p4 _setup_vars _error_handler _reset _redir_i _redir_o _form_vals _write_file
 Info Label Get ci Checkin co Checkout User Group Client Protect
 getForm setForm Admin Ulist SList
);

# @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION
use vars qw(
 %p4db %MUL $options $arguments %form $formFile $result
 $datapath $licenses $userlist
 $p4Schema $p4XSLT
  $VERSION *SAVEIN *SAVEOUT *SAVEERR
);

$VERSION = '0.18';

my $tempdir = (defined $ENV{tmp} && -d $ENV{tmp} && -w $ENV{tmp}) ?
				$ENV{tmp} : 
				(defined $ENV{TEMP} && -d $ENV{TEMP}) ?
					$ENV{TEMP} :
					'.'; # use env var tmp or TEMP or . (current dir)


BEGIN {
	# ini conf variables - server aliases
	# default directory for data files - p4-servers.ini & MUL.txt
	$datapath = ".";
	$licenses = 395;
	$userlist = -e './MUL.txt' ? './MUL.txt' : 'nul';

	my @sects;
	my $profile = -e './p4servers.ini' ? './p4servers.ini' : 'nul';
	if (-e $profile) {
		require 'IniConf.pm';
		import IniConf qw(new);

		$result = "Using active profile, $profile";
		my $cfg = IniConf->new(-file=>$profile);

		if (defined $cfg) {
			@sects = $cfg->Sections;
		} else {
			print "Invalid data file: aliases disabled for this session\n";
			foreach (@IniConf::errors) {print};
		}

		foreach my $sect (@sects) {
			foreach my $envk ($cfg->Parameters($sect)) {
				$p4db{$sect}{$envk} = $cfg->val($sect, $envk);
			}
		}
	} else {
		$result = "Failed to find profile, $profile";
	}
}

$formFile = "$tempdir/form.p4";
_reset();

# the ini file translation to XML syntax
$p4Schema =<<'EOS';
<?xml version='1.0' encoding='iso-8859-1'?>
<Schema name='perforce'
	xmlns='urn:schemas-microsoft-com:xml-data'
	xmlns:dt='urn:schemas-microsoft-com:datatypes'>

<!--
	Copyright 2000-2001 Cognos Inc.
	http://wottcopej2k/
	This Schema is based on XML-Schema support found in
	Microsoft Internet Explorer 5.
-->

<!-- id and value are shared by many elements -->
<!-- value is required in most but not all elements it appears in -->
<AttributeType name='id' required='yes' dt:type='string' />

<ElementType name='name' content='eltOnly' />
<ElementType name='description' content='eltOnly' />
<ElementType name='status' content='eltOnly' />
<ElementType name='contact' content='eltOnly' />
<ElementType name='admin' content='eltOnly' />
<ElementType name='P4HOST' content='eltOnly' />
<ElementType name='P4PORT' content='eltOnly' />
<ElementType name='P4ROOT' content='eltOnly' />
<ElementType name='P4LOG' content='eltOnly' />
<ElementType name='P4JOURNAL' content='eltOnly' />

<ElementType name='server' content='eltOnly' model='closed' >
   <attribute type='id' required='yes' />
   <element type='name' minOccurs='1' maxOccurs='1' />
   <element type='description' minOccurs='1' maxOccurs='1' />
   <element type='status' minOccurs='1' maxOccurs='1' />
   <element type='contact' minOccurs='1' maxOccurs='1' />
   <element type='admin' minOccurs='1' maxOccurs='1' />
   <element type='P4HOST' minOccurs='1' maxOccurs='1' />
   <element type='P4PORT' minOccurs='1' maxOccurs='1' />
   <element type='P4ROOT' minOccurs='1' maxOccurs='1' />
   <element type='P4LOG' minOccurs='1' maxOccurs='1' />
   <element type='P4JOURNAL' minOccurs='1' maxOccurs='1' />
</ElementType>

<ElementType name='perforce' content='eltOnly' model='closed' order='one' >
   <group order='many' >
	   <element type='server' minOccurs='0' maxOccurs='*' />
   </group>
</ElementType>

</Schema>
EOS

$p4XSLT =<<'EOX';
<?xml version='1.0' encoding='iso-8859-1'?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:perforce="x-schema:perforce-Schema.xml"
	xmlns:auto-ns2="http://www.w3.org/TR/xhtml1/strict"
	xmlns="http://www.w3.org/1999/xhtml">

	<!-- [XSL-XSLT] Explicitly apply the default (and only) indent-result behavior -->
	<xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>

	<!-- [XSL-XSLT] Simulate lack of built-in templates -->
	<xsl:template match="@*|/|node()"/>
	<!-- top 'O doc -->
	<xsl:template match="/"><html lang="en" xml:lang="en" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:auto-ns2="http://www.w3.org/TR/xhtml1/strict" xmlns="http://www.w3.org/1999/xhtml">

	<head>

	<title>Perforce Servers</title>

	<!--
		Copyright 2000-2001 Cognos Inc.
		http://wottcopej2k
		This XML doc was produced from XSL stylesheets based on
		support found in Microsoft Internet Explorer 5.5 with MSXML3
	-->

	<style type="text/css">
		body { color: black;
			background: cyan; }
		h1 { text-align: center;
			color: blue;
			background: yellow; }
		table {	border: solid darkred 3px;
			border-collapse: separate;
			border-spacing: 5px;
			empty-cells: show; }
		tr { text-align: center;
			vertical-align: top;
			background-color: white;
			color: black;}
		td {	border: ridge darkred 3px;
			empty-cells: show; }
		:link { color: blue; }
		:active { color: white; }
		:visited { color: silver; }
		a:hover { background-color: yellow; }
	</style>

	<meta name="GENERATOR" content="Copestetics (Win2K; M) [Cognos]" />
	<meta http-equiv='Content-Type' content='text/html; charset=iso-8859-1' />

	</head>

	<body xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/TR/xhtml1/strict">
	<h1>Perforce Servers</h1>
	<xsl:apply-templates select="perforce" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:perforce="x-schema:perforce-Schema.xml"></xsl:apply-templates>
	</body>

	</html>
	</xsl:template>
	<xsl:template match="server" xmlns:perforce="x-schema:perforce-Schema.xml">
		<tr>
		<td><xsl:value-of select="@id" /></td>
		<td><xsl:value-of select="name" /></td>
		<td><xsl:value-of select="description" /></td>
		<td><xsl:value-of select="status" /></td>
		<td><xsl:value-of select="contact" /></td>
		<td><xsl:value-of select="admin" /></td>
		<td><xsl:value-of select="P4HOST" />:<xsl:value-of select="P4PORT" /></td>
		<td><xsl:value-of select="P4ROOT" /></td>
		<td><xsl:value-of select="P4LOG" /></td>
		<td><xsl:value-of select="P4JOURNAL" /></td>
		</tr>
	</xsl:template>
	<xsl:template match="perforce" xmlns:perforce="x-schema:perforce-Schema.xml">
		<table>
		<th>
		<tr>
		<td>ID</td>
		<td>Name</td>
		<td>Description</td>
		<td>Status</td>
		<td>Contact</td>
		<td>Admin</td>
		<td>Host:Port</td>
		<td>Root</td>
		<td>Log</td>
		<td>Journal</td>
		</tr>
		</th>
		<xsl:apply-templates select="server" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"></xsl:apply-templates>
		</table>
	</xsl:template>
</xsl:stylesheet>
EOX

sub run_p4 {
#####
	# runs a Perforce command (p4)
#####
	my ($cmd) = @_;
	$result = undef;

	open(SUBPROC, "p4 $options $cmd $arguments |")
		or return $!;

	local($/) = undef;
	my $ret = <SUBPROC>;
	close SUBPROC;
	if ($?) {
	   	_error_handler("$cmd $options $arguments", $ret);
		_reset();
		return;
	} else {
		$result = $ret;
	}
	_reset();
	return $ret;
}

sub _setup_vars {
#####
	# given options hash, sets the package global variables, deletes the form file
#####
	my %opts = @_;

	$options .= " -p " . $p4db{$opts{alias}}{P4HOST} . ':' . $p4db{$opts{alias}}{P4PORT} if (exists $opts{alias} && defined $opts{alias});
	$options .= $opts{options} if defined $opts{options};
	$arguments .= $opts{arguments} if defined $opts{arguments};

	chmod 0755, $formFile;
	unlink $formFile;
}

sub _error_handler {
#####
	# stores result code in $result and prints error message
#####
	my $particulars = shift;
	# last arg is either scalar or array

	$result = $? >> 8;
	carp("$0: '$particulars' result code: '$result'\n");
}

sub _reset {
#####
	# resets package global session variables
#####
	$options = "";
	$arguments = "";
}

sub _redir_i {
#####
	# given a redirect method (open or close) and form parameters
	# creates the form file and redirects STDIN from the form file 
#####
	my $arg = shift;
	my (%opts, %params);
    if (@_) {
		%params = setForm(@_);
    }

	if ($arg eq 'open') {
		if (exists $params{form}) {
			my $data;

			foreach my $key (keys %{$params{form}}) {
				next if $key =~ /^(Update|Access)/;
				if (ref($params{form}{$key}) =~ /ARRAY/) {
					$data .= "$key:\n";
					foreach my $item (@{$params{form}{$key}}) {
						$data .= "\t$item\n";
					}
				} else {
					$data .= "$key: $params{form}{$key}\n" if defined $params{form}{$key};
				}
			}
			_write_file($formFile, $data);
		}

		if (-e $formFile) {
			open(SAVEIN, "<&STDIN");
			open(STDIN, "<$formFile");
		} else {
			return;
		}
	} else { # assume close
		close(STDIN);
		open(STDIN, "<&SAVEIN");
	}
	select STDIN;
	$| = 1;
	select STDOUT;
	return 1;
}

sub _redir_o {
#####
	# given a redirect method (open or close), redirects STDOUT
	# to the form file
	# THIS JUST DOESN'T WORK AT ALL - use the return val from run_p4 instead:
	#         turn it into an array (run_p4 returns scalar only) - split /\n/
	#         run it through _form_vals('set', @result)
#####
	my ($opts) = @_;

	if ($opts eq 'open') {
# replacement code
#		@list = split /\n/, run_p4('command');

		open(SAVEOUT, ">&STDOUT");
		open(STDOUT, ">$formFile");
		open SAVEERR, ">STDERR";
		open STDERR, ">&STDOUT";
	} else { # assume close
# replacement code
#		_form_vals('get', @list);

		close(STDOUT);
		open(STDOUT, ">&SAVEOUT");
		close STDERR;
		open STDERR, ">&SAVEERR";

		select STDOUT;
		$| = 1;
		select STDIN;

		# read back form values from file into data structure
		_redir_i('open');
		_form_vals('get', <STDIN>);
		_redir_i('close');
	}
	select STDOUT;
	$| = 1;
	select STDIN;
	return 1;
}

# form interface
sub setForm {
#####
	# given any of form => $ref, form => {} or %form, 
	# sets global %form and returns copy of same
#####
	my %params;
	shift if ($_[0] =~ /form/i || ref($_[1]));

	%form = %{$_[0]};
	return {form => %form};
}

sub getForm {
#####
	# returns global form as %form
#####
	return {form => %form};
}

sub _form_vals {
#####
	# Given an $opt (get or set), reads form datastructure or
	# writes the form.p4 file (since that doesn't work, pass the return
	# value, converted from string to array, of p4 form gen'd command)
#####
	my $opt = shift;
	my @form = @_;# || <>;
	my ($tag, $val);

	if ($opt =~ /get/) {
		undef %form;
		foreach (@form) {
			chomp;
			next if m!^$!;
			next if m!^#!;
			$val = undef;

			if (m!^(.[^\s]*):\s*(.*)!) {
				($tag, $val) = ($1, $2);
			} elsif (m!^(.[^\s]*):\s*$!) {
				$tag = $1;
				next;
			} else {
				($val = $_) =~ s!\s*(.*)\s*$!$1!;
			}

			if (exists $form{$tag}) {
				if (ref $form{$tag} eq 'ARRAY') {
					push @{$form{$tag}}, $val;
				} else {
					$form{$tag} .= $val;
				}
			} else {
				if ($val) {
					$form{$tag} = $val;
				} else {
					@{$form{$tag}} = ();
				}
			}
		}
	} else {
		my $data;
		foreach my $key (keys %{$form{form}}) {
			if (ref($form{form}->{$key}) =~ /ARRAY/) {
				$data .= "$key:\n";
				foreach my $item (@{$form{form}->{$key}}) {
					$data .= "\t$item\n";
				}
			} else {
				$data .= "$key: $form{form}->{$key}\n" if defined $form{form}->{$key};
			}
		}
		_write_file($formFile, $data);
	}
}

sub _write_file {
#####
	# given name and data, writes data to file named name optionally as binary
#####
	my ($name, $data, $flag) = @_;

	if (-e $name) {chmod 0777, $name; unlink $name}
    open OUT, ">$name";
	binmode OUT if defined $flag; # make it so by default
    print OUT $data;
    close OUT;
}

sub Info {
#####
	# Given a dummy arg, returns perforce Information
#####
	my $arg = shift;
	my %opts = @_;

	_setup_vars(%opts);

	run_p4('info');
}

sub Label {
#####
	# given label and params, either in form hash or 
	# named vals of a form to be made hash, applies label
#####
	my $label = shift;
	my %opts = @_;

	_setup_vars(%opts);
	$arguments .= ' -i ';

# TODO:
# should ensure unique label name (labels are in the same namespace as client and depot)
# when locked option is set: if label isn't available, declare new, sync then lock it otherwise, just apply the label
	if (exists $opts{form}) {
		_redir_i('open', form => $opts{form}) or return undef;
	} else {
		$form{Label} = $label;
		$form{Owner} = $opts{Owner} if exists $opts{Owner};
		@{$form{Description}} = @{$opts{Description}} if (exists $opts{Description} && defined @{$opts{Description}});
		$form{Options} = $opts{Options} if exists $opts{Options};
		@{$form{View}} = @{$opts{View}} if (exists $opts{View} && defined @{$opts{View}});
		_redir_i('open', %form);
	}

	my $return = run_p4('label');
	_redir_i('close');

	_setup_vars(%opts);
	$arguments = " -l $label ";
	$return .= run_p4('labelsync') if ($result && "\slocked" !~ /@{$form{View}}/);
	$return;
}

sub Get {
#####
	# given filename as spec, performs sync
#####
	my $file = shift;
	my %opts = @_;

	_setup_vars(%opts);
	$arguments .= " $file ";

	run_p4('sync');
}

sub Flush {
#####
	# given filename as spec (or dummy arg), performs flush
#####
	my $file = shift;
	my %opts = @_;

	_setup_vars(%opts);
	$arguments .= " $file ";

	run_p4('flush');
}

sub co { Checkout(@_) }
sub Checkout {
#####
	# given filename, executes edit
#####
	my $file = shift;
	my %opts = @_;

	$arguments = " $file ";
	_setup_vars(%opts);

	run_p4('edit');
}

sub ci { Checkin(@_) }
sub Checkin {
#####
	# given filename as item, submits changelist item
#####
	my $file = shift;
	my %opts = @_;

	_setup_vars(%opts);
	$arguments .= " -i $file ";

	if (exists $opts{form}) {
		_redir_i('open', $opts{form}) or return undef;
	} else {
return; # force error, unfinished work
		_redir_i('open',  
			this => $opts{that}
			);
	}

	my $return = run_p4('submit');
	_redir_i('close');

	$return;
}

sub Client {
#####
	# given an arg (get, set or use) manipulates P4 client record
#####
	my $arg = shift; # add or del
	my %opts = @_;
	my ($return, @list);

	_setup_vars(%opts);

	if ($arg =~ /get/i) {
		$arguments .= " -o ";
		@list = split /\n/, run_p4('client');
		_form_vals('get', @list);
		$return = \%form;
	} elsif ($arg =~ /set/) {
		$arguments = " -i ";
		$arguments .= $opts{arguments} if defined $opts{arguments};
		if (exists $opts{form}) {
			_redir_i('open', $opts{form});
		} else {
			$form{Owner} = $opts{Owner} if exists $opts{Owner};
			$form{Host} = $opts{Host} if exists $opts{Host};
			@{$form{Description}} = @{$opts{Description}} if (exists $opts{Description} && defined @{$opts{Description}});
			$form{Root} = $opts{Root} if exists $opts{Root};
			$form{Options} = $opts{Options} if exists $opts{Options};
			@{$form{View}} = @{$opts{View}} if (exists $opts{View} && defined @{$opts{View}});
			_redir_i('open', %form) or return undef;
		}
		$return = run_p4('client');
		_redir_i('close');
	} elsif ($arg =~ /use/) {
		$ENV{P4CLIENT} = $opts{Client};
	} else {
		return;
	}

	$return;
}

sub User {
#####
	# given an arg (add, del or list) manipulates P4 user record
#####
	my $arg = shift; # add or del
	my %opts = @_;
	my ($return, %list);

	$arguments = " -d $opts{User} " if $arg =~ /del/i;
	_setup_vars(%opts);

	if ($arg =~ /add/i) {
# check user against users list, only add new
#		return unless $userlist = _run_p4('users');
#		map return 'notnew' if $form{User} =~ m/^$_\s/, foreach csv in $userlist, ;
		if (exists $opts{form}) {
			_redir_i('open', $opts{form}) or return undef;
		} else {
			_redir_i('open', 
				User 	=> $opts{User},
				FullName => $opts{FullName},
				Email	=> $opts{Email}
				) or return undef;
		}

		$arguments .= ' -i ';
		$return = run_p4('user');
		_redir_i('close');
	} elsif ($arg =~ /del/) {
		$return = run_p4('user');
	} elsif ($arg =~ /list/) {
		map { $list{$1} = $2 if m/^(.[^\s]*)\s.*\s\((.[^\s]*)\)\s.*$/ } (split /\n/, run_p4('users'));
		return %list if defined %list;
	}

	$return;
}

sub Group {
#####
	# given an arg (add or del) manipulates P4 group record
#####
	my $arg = shift; # add or del
	my %opts = @_;
	my ($return, @list);

	_setup_vars(%opts);

	if ($arg =~ /add/i) {
		$arguments .= "  -o $opts{Group} ";
		@list = split /\n/, run_p4('group');
		_form_vals('get', @list);

		foreach my $user (@{$opts{Users}}) {
			next if grep(/$user/, @{$form{Users}});
			push @{$form{Users}}, $user;
		}

		$arguments .= $opts{arguments} if defined $opts{arguments};
		$arguments .= " -i ";
		_redir_i('open', 
			Group => $opts{Group},
			Users => [@{$form{Users}}]
			) or return undef;
		$return = run_p4('group');
		_redir_i('close');
	} elsif ($arg =~ /del/) {
		$arguments = " -d $opts{Group} ";
		$arguments .= $opts{arguments} if defined $opts{arguments};
		$return = run_p4('group');
	} else {
		return;
	}

	$return;
}

sub Protect {
# work needed, set only unique masks
#####
	# given an option (get or set), performs protection manip
#####
	my $arg = shift; # get or set
	my %opts = @_;
	my ($return, @list);

	_setup_vars(%opts);

	if ($arg =~ /set/) {
		foreach my $prots (@{$opts{Protections}}) {
			next if grep(quotemeta($prots), @{$form{Protections}});
			push @{$form{Protections}}, $prots;
			_redir_i('open',
				Protections => [@{$form{Protections}}]
				) or return undef;
			$arguments .= ' -i ';
			$return = run_p4('protect');
			_redir_i('close');
		}
	} elsif ($arg =~ /get/) {
		$arguments .= ' -o ';
		@list = split /\n/, run_p4('protect');
		_form_vals('get', @list);
	}

	$return;
}

sub UList {
#####
	# given an option argument (get or set), counts and records the p4 users,
	# returns the number of new records or zero for set option.
#####
	my $arg = shift; # get or set
	my ($id, $name, $data, @list, %users);
	my $return = 0;

	if ($arg =~ /get/) {
		if (-e $userlist && $userlist ne 'nul') {
			open IN, $userlist or croak "can't open $userlist";
			while (<IN>) {
				chomp;
				next if /^#/;
				($id, $name) = split /,/;
				$MUL{$id} = $name;
			}
			close IN;
		}

		foreach my $alias (keys %p4db) {
			_setup_vars(alias => $alias);
			@list = split /\n/, run_p4('users');
			map { $users{$1} = $2 if m/^(.[^\s]*)\s.*\s\((.[^\)]*)\)\s.*$/ } @list;
		}

		foreach my $key (keys %users) {
			unless (exists $MUL{$key}) {
				$MUL{$key} = $users{$key};
		        $return++;
		    }
		}
	} elsif ($arg =~ /set/) {
		$return = (keys %MUL);
		$data = "# Perforce Master User List - all $return licensed users of $licenses licenses\n";
		foreach my $key (sort keys %MUL) { $data .= $key . ',' . $MUL{$key} . "\n"}
		_write_file($userlist, $data);
	} elsif ($arg =~ /verify/) {
		$return = $licenses - (keys %MUL);
	}

	$return;
}

sub SList {
#####
	# Given an option argument (new, del, edit or report), manipulates the p4servers.ini
#####
	my $arg = shift;
	my %opts = @_ if defined @_;
	my $return = 1;

	if ($arg eq 'report') {
		my $data = "<?xml-stylesheet type='text/xsl' href='report.xslt' version='1.0' encoding='iso-8859-1'?>\n";
		$data .= "<perforce xmlns:perforce='x-schema:perforce-Schema.xml'>\n";
		foreach my $server (keys %p4db) {
			$data .= "\t<server id='$server'>\n";
			foreach my $key (keys %{$p4db{$server}}) {
				$data .= "\t\t<$key>$p4db{$server}{$key}</$key>\n";
			}
			$data .= "\t</server>\n";
		}
		$data .= "</perforce>\n";
		_write_file("$datapath/perforce.xml", $data);
		_write_file("$datapath/report.xslt", $p4XSLT);
		_write_file("$datapath/perforce-Schema.xml", $p4Schema);
	} elsif ($arg =~ /new/) {
		$return = 0;
	} elsif ($arg =~ /del/) {
		$return = 0;
	} elsif ($arg =~ /edit|update/) {
		$return = 0;
	} else {
		$return = 0;
	}

	$return;
}

sub Admin {
#####
	# given an option (stop or checkpoint), performs admin function using option
#####
	my $arg = shift;
	my %opts = @_;

	_setup_vars(%opts);

	if ($arg =~ /stop/) {
		run_p4('admin stop');
	} elsif ($arg =~ /checkpoint/) {
		run_p4('admin checkpoint');
	} else {
		return;
	}
}

1;
__END__

# perldoc!

=head1 NAME

P4 - Perl extension for Perforce, the SCM.

=head1 SYNOPSIS

  use P4;

  $options = "-p perforce:1666";
  $arguments = "";

  print info();
  print depots();
  print files();
  print jobs();
  print labels();
  print users();

  $arguments = "close";
  print admin();

Alternatively, use the common p4 commands as mapped through P4::Simple (identified as not being a proper P4 command
- usually upper cased first character). All these functions take a single action argument and optionally
a hash of options, arguments and form values.

 use P4;

 Info('dummy', options=>'-p bldsrv:1666', arguments=>'');
 Label('label', options=>'-p bldsrv:1666', arguments=>'', form => %form);
 Get('file', options=>'-p bldsrv:1666', arguments=>'');
 Checkout('file', options=>'-p bldsrv:1666', arguments=>'', form => %form);
 Checkin('file', options=>'-p bldsrv:1666', arguments=>'');

=head1 DESCRIPTION

Methods to use in a Perforce SCM. This module supports B<p4> on a rudimentary level, it assumes
the methods called are actual B<p4> commands. The P4::Simple package provides a comprehensive
set of methods to the most common P4 actions such as edit, sync and label through well known
named method verbs like Get, ci or Checkin, co or Checkout and Label. All methods return a value
on success, undefined on error and prints a message with the error code. The error code is also
stored in $P4::Simple::result.

The SList function depends on an ini file called p4servers.ini to provide information about all known
perforce servers available for use. The structure is as follows:

=over 4

 [LOCAL]
 name=Local p4 db
 P4HOST=hostname
 P4PORT=1666
 P4ROOT=/p4db
 P4LOG=/logs/p4.log
 P4JOURNAL=/logs/p4journal.log
 contact=A. Name
 status=Up
 admin=Another Name
 description=test db

=back

Provided p4servers.ini exists, a client parameter called alias can also be used in calls to the modules methods. Replace the
-p host:port construct of "options" and use the alias => key instead by setting it's value to
the ini section (the [LOCAL] part of the template shown above) of a server description. eg.
Get('file', alias => 'LOCAL');

=head1 METHODS

These are the P4::Simple methods.

=over 4

=item B<setForm(%form)>

Given a hash, form hash or a reference to a hash as the only form val, sets
the global %form data structure.

eg. %form = setForm(%form);

=item B<getForm()>

Returns the package global form structure as a form hash.

eg. %form = getForm();

=item B<Info('dummy_arg')>

Returns perforce Information.

=item B<Label('lbl', %form)>

Given label and params, either in form hash or named vals of a form, applies label.

eg. Label('lbl', Label => 'lbl', Options => 'locked');

=item B<Get('filename', %form)>

Given filename(s) as spec, performs sync. Files with spaces need to be quoted. A file
spec with date needs to be quoted and if the file spec contains a space, the quote
must contain both the file and the date. eg.

Get('"file with space.txt" "read me.txt@2001/01/20 23:59:59"', %form);

=item B<Flush('filename|dummy', %opts, %args)>

Updates a client workspace's have list without actually copying any files.

=item B<Checkin('filename', %form)>

Alias ci. Given filename as item, submits changelist item.

=item B<Checkout('filename', %form)>

Alias co. Given filename, executes edit.

=item B<Client('arg', %form)>

Given an arg (get, set or use), manipulates P4 client record.

=item B<User('arg', %form)>

Given an arg (add, del or list) manipulates P4 user record. The arg 'list' will
cause the return value to be a hash of usernames as keys and FullNames as values.

=item B<Group('arg', %form)>

Given an arg (add or del) manipulates P4 group record.

=item B<Protection('option', %form)>

given an option (get or set), performs protection manip.

=item B<Admin('option', %form)>

Given an option (stop or checkpoint), performs admin function using option.

=item B<UList('option')>

Given an option (get, set or verify), counts or updates the Master User List (MUL - a text
file of csv delimited user ids and thier associated names) for all servers listed in
the p4servers.ini config file. Returns the number of new user accounts not listed in the
MUL (issue 'set' to update if so: eg. C<UList('set') if UList('get')>)
or the number of valid licenses remaining after counting the licenses used
($P4::Simple::license defaults to 395).

=item B<SList('option')>

Given an option (new, del, update (or edit) or report), manips p4servers.ini.

=back

This is the list of methods that perforce understands as commands. Arguments to these commands
can be found in the Perforce documentation. Use these commands through the AUTOLOAD facility
of the P4 module (set $options and $arguments as appropriate) by specifying the command verb
alone. ie. info() will be passed to P4, wrapped in a p4 command with any $options and $arguments
defined in the package within which the info() call is made.
eg. perl -MP4 -e "$options='-p wottcopej2k:1666';$arguments='';print info();"

=over 4

=item B<add>

Open a new file to add it to the depot.

=item B<admin>

Perform administrative operations on the server.

=item B<branch>

Create or edit a branch specification.

=item B<branches>

Display list of branches.

=item B<change>

Create or edit a changelist description.

=item B<changes>

Display list of pending and submitted changelists.

=item B<client>

Create or edit a client specification and its view.

=item B<clients>

Display list of known clients.

=item B<counter>

Display, set, or delete a counter.

=item B<counters>

Display list of known counters.

=item B<delete>

Open an existing file to delete it from the depot.

=item B<depot>

Create or edit a depot specification.

=item B<depots>

Display list of depots.

=item B<describe>

Display a changelist description.

=item B<diff>

Display diff of client file with depot file.

=item B<diff2>

Display diff of two depot files.

=item B<dirs>

List subdirectories of a given depot directory.

=item B<edit>

Open an existing file for edit.

=item B<filelog>

List revision history of files.

=item B<files>

List files in the depot.

=item B<fix>

Mark jobs as being fixed by named changelists.

=item B<fixes>

List what changelists fix what job.

=item B<flush>

Fake a 'p4 sync' by not moving files.

=item B<fstat>

Dump file info.

=item B<group>

Change members of a user group.

=item B<groups>

List groups (of users).

=item B<have>

List revisions last synced.

=item B<help>

Print this help message.

=item B<info>

Print out client/server information.

=item B<integrate>

Schedule integration from one file to another.

=item B<integrated>

Show integrations that have been submitted.

=item B<job>

Create or edit a job (defect) specification.

=item B<jobs>

Display list of jobs.

=item B<jobspec>

Edit the job template.

=item B<label>

Create or edit a label specification and its view.

=item B<labels>

Display list of labels.

=item B<labelsync>

Synchronize label with the current client contents.

=item B<lock>

Lock an opened file against changelist submission.

=item B<obliterate>

Remove files and their history from the depot.

=item B<opened>

Display list of files opened for pending changelist.

=item B<passwd>

Set the user's password on the server (and Windows client).

=item B<print>

Retrieve a depot file to the standard output.

=item B<protect>

Modify protections in the server namespace.

=item B<rename>

Explains how to rename files.

=item B<reopen>

Change the type or changelist number of an opened file.

=item B<resolve>

Merge open files with other revisions or files.

=item B<resolved>

Show files that have been merged but not submitted.

=item B<revert>

Discard changes from an opened file.

=item B<review>

List and track changelists (for the review daemon).

=item B<reviews>

Show what users are subscribed to review files.

=item B<set>

Set variables in the registry (Windows only). Considered environment variables.

=item B<submit>

Submit open files to the depot.

=item B<sync>

Synchronize the client with its view of the depot.

=item B<triggers>

Modify list of pre-submit triggers.

=item B<unlock>

Release a locked file but leave open.

=item B<user>

Create or edit a user specification.

=item B<users>

Display list of known users.

=item B<verify>

Verify that the server archives are intact.

=item B<where>

Show how file names map through the client view.

=back

=head1 PROPERTIES

=over 4

=item * B<options:>

=item B<-c> <client>

Specifies the client workspace. Overrides environment variable B<P4CLIENT>.

=item B<-d> <dir>

Specifies the directory. Overrides environment variable B<PWD>.

=item B<-H> <host>

Specifies the host. Overrides environment variable B<P4HOST>.

=item B<-p> <port>

Specifies a perforce host and port as host:port. Overrides environment variable B<P4PORT>.

=item B<-P> <pass>

Specifies the password. Overrides environment variable B<P4PASSWD>.

=item B<-s> <>

Explanatory tags accompany output reports

=item B<-u> <user>

Specifies the user name. Overrides environment variable B<P4USER>, B<USER> or B<USERNAME>.

=item B<-x> <file>

Specifies a file in which each line is an argument. A hyphen indicates reading from STDIN (don't
do this in an unattended, non-interactive or otherwise automated script).

=item * B<arguments:>

Whatever is appropriate for the command such as a changlist number or file name and switches.

=item * B<Notes:>

Each method follows a standard calling convention. All methods are Perforce commands
and will be called as "B<p4> B<options> B<command> B<arguments>". The package methods
follow this convention formulating a command with it's properties (options and arguments).
The return value is either error text or B<p4> output. It is incombent on the user to call
submit for the object methods that require submission.

=back

=head1 AUTHOR

John G. Cope, John.Cope@Cognos.Com

=head1 SEE ALSO

perl (1) and B<p4> help.

=cut
